function ShortcutDecorator(ApplicationContext, templateManager) {

    /**
     *  Default keys for CRUD icons
     */
    const defaultCRUDShortcutKeys = {
        "add": "mod+a",
        "edit": "mod+e",
        "delete": "mod+d"
    };

    /**
     * 
     * Get the 'mod' key name according to OS (Ctrl or Command)
     * @returns String(Ctrl, Command) / Boolean(false)
     */
    function getTooltipModKey() {
        var platform = window.navigator.platform,
            commandPlatforms = ['Macintosh', 'MacIntel', 'MacPPC', 'Mac68K'],
            ctrlPlatforms = ['Win32', 'Win64', 'Windows', 'WinCE'],
            mod = false;

        if (commandPlatforms.indexOf(platform) !== -1) {
            mod = 'Command + ';
        } else if (ctrlPlatforms.indexOf(platform) !== -1 || /Linux/.test(platform)) {
            mod = 'Ctrl + ';
        }
        return mod;
    }

    /**
     * 
     * Get shortcut key from shortcutKeys object of widget, if not defined, 
     * from project property and if not defined, 
     * from defaultCRUDShortcutKeys const in this function.
     * @param {any} widget 
     * @param {any} shortcut 
     * @returns String(Shortcut key) / Undefined
     */
    function getShortcutKey(widget, shortcut) {
        return (
            Util.getNestedProperty(widget, ['shortcutKeys', shortcut]) ||
            Util.getNestedProperty(templateManager, ['project', 'shortcutKeys', shortcut]) ||
            defaultCRUDShortcutKeys[shortcut]
        );
    }
/**
 * 
 * Iterate recursively in widgets of container applying shortcut key in actions. 
 * @param {any} widgets 
 */
function applyShortcutOnAction(widgets) {
        var modTooltip = getTooltipModKey();
        widgets.forEach(function (widget) {
            if (widget.actions) {
                widget.actions.forEach(function (action) {
                    if (action.icon === 'plus' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'add');
                    } else if (action.icon === 'pencil' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'edit');
                    } else if (action.icon === 'trash' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'delete');
                    } else if (!action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, action.icon);
                    }
                    if (modTooltip && action.shortcutKey && !action.tooltip && templateManager.project.showShortcutKeyAsTooltip !== false) {
                        action.tooltip = action.shortcutKey.toUpperCase().replace('MOD+', modTooltip);
                    }
                });
            }
            if (widget.widgets && widget.widgets.length) {
                applyShortcutOnAction(widget.widgets);
            }
        });
    }

    /**
     * 
     * If project have the property enableCRUDShortcutKeys == true, applies the shortcut keys in actions
     * @param {any} containerName 
     * @param {any} container 
     * @returns Container(container)
     */
    this.apply = function (containerName, container) {
        if (templateManager.project.enableCRUDShortcutKeys == true) {
            applyShortcutOnAction(container.widgets);
        }
        return container;
    };

}

Configuration(function (ContainerDecoratorProvider, ContextRegister, ApplicationContext, templateManager) {
    ContextRegister.register('ShortcutDecorator', ShortcutDecorator);
    ContainerDecoratorProvider.registerProvider(new ShortcutDecorator(ApplicationContext, templateManager));
});
